@@ -12,7 +12,7 @@ class Redis_Cluster_Test extends Redis_Test {
     private $_arr_redis_types = [
         Redis::REDIS_STRING,
         Redis::REDIS_SET,
-        Redis::REDIS_LIST,  
+        Redis::REDIS_LIST,
         Redis::REDIS_ZSET,
         Redis::REDIS_HASH
     ];
@@ -33,7 +33,7 @@ class Redis_Cluster_Test extends Redis_Test {
      */
     protected $sessionSaveHandler = 'rediscluster';
 
-    /* Tests we'll skip all together in the context of RedisCluster.  The 
+    /* Tests we'll skip all together in the context of RedisCluster.  The
      * RedisCluster class doesn't implement specialized (non-redis) commands
      * such as sortAsc, or sortDesc and other commands such as SELECT are
      * simply invalid in Redis Cluster */
@@ -46,6 +46,8 @@ class Redis_Cluster_Test extends Redis_Test {
     public function testDoublePipeNoOp() { return $this->markTestSkipped(); }
     public function testSwapDB() { return $this->markTestSkipped(); }
     public function testConnectException() { return $this->markTestSkipped(); }
+    public function testTlsConnect() { return $this->markTestSkipped(); }
+    public function testInvalidAuthArgs() { return $this->markTestSkipped(); }
 
     /* Session locking feature is currently not supported in in context of Redis Cluster.
        The biggest issue for this is the distribution nature of Redis cluster */
@@ -62,7 +64,9 @@ class Redis_Cluster_Test extends Redis_Test {
     public function testSession_lockWaitTime() { return $this->markTestSkipped(); }
 
     /* Load our seeds on construction */
-    public function __construct() {
+    public function __construct($str_host, $i_port, $str_auth) {
+        parent::__construct($str_host, $i_port, $str_auth);
+
         $str_nodemap_file = dirname($_SERVER['PHP_SELF']) . '/nodes/nodemap';
 
         if (!file_exists($str_nodemap_file)) {
@@ -87,7 +91,7 @@ class Redis_Cluster_Test extends Redis_Test {
 
     /* Override newInstance as we want a RedisCluster object */
     protected function newInstance() {
-        return new RedisCluster(NULL, self::$_arr_node_map);
+        return new RedisCluster(NULL, self::$_arr_node_map, 30, 30, true, $this->getAuth());
     }
 
     /* Overrides for RedisTest where the function signature is different.  This
@@ -220,6 +224,55 @@ class Redis_Cluster_Test extends Redis_Test {
         $this->assertEquals($i_scan_count, $i_key_count);
     }
 
+    public function testScanPrefix() {
+        $arr_prefixes = ['prefix-a:', 'prefix-b:'];
+        $str_id = uniqid();
+
+        $arr_keys = [];
+        foreach ($arr_prefixes as $str_prefix) {
+            $this->redis->setOption(Redis::OPT_PREFIX, $str_prefix);
+            $this->redis->set($str_id, "LOLWUT");
+            $arr_keys[$str_prefix] = $str_id;
+        }
+
+        $this->redis->setOption(Redis::OPT_SCAN, Redis::SCAN_RETRY);
+        $this->redis->setOption(Redis::OPT_SCAN, Redis::SCAN_PREFIX);
+
+        foreach ($arr_prefixes as $str_prefix) {
+            $arr_prefix_keys = [];
+            $this->redis->setOption(Redis::OPT_PREFIX, $str_prefix);
+
+            foreach ($this->redis->_masters() as $arr_master) {
+                $it = NULL;
+                while ($arr_iter = $this->redis->scan($it, $arr_master, "*$str_id*")) {
+                    foreach ($arr_iter as $str_key) {
+                        $arr_prefix_keys[$str_prefix] = $str_key;
+                    }
+                }
+            }
+
+            $this->assertTrue(count($arr_prefix_keys) == 1 && isset($arr_prefix_keys[$str_prefix]));
+        }
+
+        $this->redis->setOption(Redis::OPT_SCAN, Redis::SCAN_NOPREFIX);
+
+        $arr_scan_keys = [];
+
+        foreach ($this->redis->_masters() as $arr_master) {
+            $it = NULL;
+            while ($arr_iter = $this->redis->scan($it, $arr_master, "*$str_id*")) {
+                foreach ($arr_iter as $str_key) {
+                    $arr_scan_keys[] = $str_key;
+                }
+            }
+        }
+
+        /* We should now have both prefixs' keys */
+        foreach ($arr_keys as $str_prefix => $str_id) {
+            $this->assertTrue(in_array("${str_prefix}${str_id}", $arr_scan_keys));
+        }
+    }
+
     // Run some simple tests against the PUBSUB command.  This is problematic, as we
     // can't be sure what's going on in the instance, but we can do some things.
     public function testPubSub() {
@@ -274,7 +327,7 @@ class Redis_Cluster_Test extends Redis_Test {
         $ret = $this->redis->msetnx(['x'=>'a','y'=>'b','z'=>'c']);
         $this->assertTrue(is_array($ret));
         $this->assertEquals(array_sum($ret),1);
-        
+
         $this->assertFalse($this->redis->msetnx(array())); // set ø → FALSE
     }
 
@@ -390,7 +443,7 @@ class Redis_Cluster_Test extends Redis_Test {
         $this->assertTrue(1 === $this->redis->eval($scr,[$str_key], 1));
         $this->assertTrue(1 === $this->redis->evalsha($sha,[$str_key], 1));
     }
-    
+
     public function testEvalBulkResponse() {
         $str_key1 = uniqid() . '-' . rand(1,1000) . '{hash}';
         $str_key2 = uniqid() . '-' . rand(1,1000) . '{hash}';
@@ -458,7 +511,7 @@ class Redis_Cluster_Test extends Redis_Test {
     public function testIntrospection() {
         $arr_masters = $this->redis->_masters();
         $this->assertTrue(is_array($arr_masters));
-        
+
         foreach ($arr_masters as $arr_info) {
             $this->assertTrue(is_array($arr_info));
             $this->assertTrue(is_string($arr_info[0]));
@@ -467,10 +520,10 @@ class Redis_Cluster_Test extends Redis_Test {
     }
 
     protected function genKeyName($i_key_idx, $i_type) {
-        switch ($i_type) { 
-            case Redis::REDIS_STRING: 
+        switch ($i_type) {
+            case Redis::REDIS_STRING:
                 return "string-$i_key_idx";
-            case Redis::REDIS_SET: 
+            case Redis::REDIS_SET:
                 return "set-$i_key_idx";
             case Redis::REDIS_LIST:
                 return "list-$i_key_idx";
@@ -491,7 +544,7 @@ class Redis_Cluster_Test extends Redis_Test {
         switch ($i_type) {
             case Redis::REDIS_STRING:
                 $value = "$str_key-value";
-                $this->redis->set($str_key, $value); 
+                $this->redis->set($str_key, $value);
                 break;
             case Redis::REDIS_SET:
                 $value = [
@@ -509,7 +562,7 @@ class Redis_Cluster_Test extends Redis_Test {
                     $str_key . '-mem3' => $str_key . '-val3'
                 ];
                 $this->redis->hmset($str_key, $value);
-                break; 
+                break;
             case Redis::REDIS_LIST:
                 $value = [
                     $str_key . '-ele1', $str_key . '-ele2', $str_key . '-ele3',
@@ -528,7 +581,7 @@ class Redis_Cluster_Test extends Redis_Test {
                 foreach ($value as $str_mem => $i_score) {
                     $this->redis->zadd($str_key, $i_score, $str_mem);
                 }
-                break; 
+                break;
         }
 
         /* Update our reference array so we can verify values */
@@ -585,7 +638,7 @@ class Redis_Cluster_Test extends Redis_Test {
         for ($i = 0; $i < 200; $i++) {
             foreach ($this->_arr_redis_types as $i_type) {
                 $str_key = $this->setKeyVals($i, $i_type, $arr_value_ref);
-                $arr_type_ref[$str_key] = $i_type;                
+                $arr_type_ref[$str_key] = $i_type;
             }
         }
 
@@ -598,7 +651,7 @@ class Redis_Cluster_Test extends Redis_Test {
             }
 
             break;
-        }    
+        }
     }
 
     /* Test a 'raw' command */
@@ -636,12 +689,19 @@ class Redis_Cluster_Test extends Redis_Test {
         $this->redis->setOption(Redis::OPT_REPLY_LITERAL, false);
     }
 
+    /* Redis and RedisCluster use the same handler for the ACL command but verify we can direct
+       the command to a specific node. */
+    public function testAcl() {
+        if ( ! $this->minVersionCheck("6.0"))
+            return $this->markTestSkipped();
+
+        $this->assertInArray('default', $this->redis->acl('foo', 'USERS'));
+    }
+
     public function testSession()
     {
         @ini_set('session.save_handler', 'rediscluster');
-        @ini_set('session.save_path', implode('&', array_map(function ($seed) {
-            return 'seed[]=' . $seed;
-        }, self::$_arr_node_map)) . '&failover=error');
+        @ini_set('session.save_path', $this->getFullHostPath() . '&failover=error');
         if (!@session_start()) {
             return $this->markTestSkipped();
         }
@@ -649,16 +709,47 @@ class Redis_Cluster_Test extends Redis_Test {
         $this->assertTrue($this->redis->exists('PHPREDIS_CLUSTER_SESSION:' . session_id()));
     }
 
+
+    /* Test that we are able to use the slot cache without issues */
+    public function testSlotCache() {
+        ini_set('redis.clusters.cache_slots', 1);
+
+        $pong = 0;
+        for ($i = 0; $i < 10; $i++) {
+            $obj_rc = new RedisCluster(NULL, self::$_arr_node_map, 30, 30, true, $this->getAuth());
+            $pong += $obj_rc->ping("key:$i");
+        }
+
+        $this->assertEquals($pong, $i);
+
+        ini_set('redis.clusters.cache_slots', 0);
+    }
+
+    /* Regression test for connection pool liveness checks */
+    public function testConnectionPool() {
+        $prev_value = ini_get('redis.pconnect.pooling_enabled');
+        ini_set('redis.pconnect.pooling_enabled', 1);
+
+        $pong = 0;
+        for ($i = 0; $i < 10; $i++) {
+            $obj_rc = new RedisCluster(NULL, self::$_arr_node_map, 30, 30, true, $this->getAuth());
+            $pong += $obj_rc->ping("key:$i");
+        }
+
+        $this->assertEquals($pong, $i);
+        ini_set('redis.pconnect.pooling_enabled', $prev_value);
+    }
+
     /**
      * @inheritdoc
      */
     protected function getFullHostPath()
     {
-        $hosts = array_map(function ($host) {
-            return 'seed[]=' . $host . '';
-        }, self::$_arr_node_map);
+        $auth = $this->getAuthFragment();
 
-        return implode('&', $hosts);
+        return implode('&', array_map(function ($host) {
+            return 'seed[]=' . $host;
+        }, self::$_arr_node_map)) . ($auth ? "&$auth" : '');
     }
 }
 ?>
